package com.jeeplus.test.reimbursementAssess.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.jeeplus.aop.demo.annotation.DemoMode;
import com.jeeplus.aop.logging.annotation.ApiLog;
import com.jeeplus.common.beanvalidator.BeanValidators;
import com.jeeplus.core.excel.ExcelOptions;
import com.jeeplus.core.excel.ExportMode;
import com.jeeplus.core.excel.utils.EasyPoiUtil;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.utils.DictUtils;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.test.reimbursementAssess.domain.dto.AssessReimbursementBusinessReturnDTO;
import com.jeeplus.test.reimbursementAssess.domain.dto.AssessReimbursementUserDTO;
import com.jeeplus.test.reimbursementAssess.service.AssessReimbursementUserService;
import com.jeeplus.test.reimbursementProject.domain.dto.ProjectReimbursementUserDTO;
import com.jeeplus.test.reimbursementsys.domain.dto.ReimbursementUserImportDTO;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.time.LocalDate;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author 徐滕
 * @version 1.0.0
 * @ClassName ReimbursementUserController.java
 * @Description TODO 报销人员年份金额信息
 * @createTime 2022年05月16日 10:35:21
 */
@RestController
@RequestMapping("/assess/reimbursementUser")
public class AssessReimbursementUserController {

    @Autowired
    private AssessReimbursementUserService service;

    /**
     * 根据id报销人员信息
     * @param id
     * @return
     */
    @ApiLog("根据id报销人员信息")
    @PreAuthorize("hasAnyAuthority('assess:reimbursementUser:view','assess:reimbursementUser:add','assess:reimbursementUser:edit')")
    @GetMapping("queryById")
    public ResponseEntity queryById(@RequestParam String id) {
        AssessReimbursementUserDTO reimbursementDTO = service.queryById (id);
        return ResponseEntity.ok (reimbursementDTO);
    }

    /**
     * 根据id报销人员信息
     * @return
     */
    @ApiLog("查询字典中日报销额度和年度假日天数")
    @PreAuthorize("hasAnyAuthority('assess:reimbursementUser:add')")
    @GetMapping("getReimbursementQuotaDay")
    public ResponseEntity getReimbursementQuotaDay() {
        AssessReimbursementUserDTO reimbursementDTO = new AssessReimbursementUserDTO();
        //获取日报销额度
        String reimbursementQuotaDay = DictUtils.getDictLabel("1", "reimbursement_quota_day", null);
        //获取当年度的总天数
        int yearDay = LocalDate.now().lengthOfYear();
        //获取年度休假天数
        String holidayDay = DictUtils.getDictLabel("1", "reimbursement_holiday_day", null);
        Integer reimbursementAllDay = yearDay;
        if(StringUtils.isNotBlank(holidayDay)){
            int i = Integer.parseInt(holidayDay);
            reimbursementAllDay = yearDay - Integer.parseInt(holidayDay);
        }
        reimbursementDTO.setReimbursementQuotaDay(reimbursementQuotaDay);
        reimbursementDTO.setReimbursementAllDay(reimbursementAllDay);
        return ResponseEntity.ok (reimbursementDTO);
    }


    /**
     * 查询报销信息
     * @return
     */
    @ApiLog("查询报销人员信息列表")
    @GetMapping("findList")
    public ResponseEntity findList(AssessReimbursementUserDTO reimbursementUser, Page<AssessReimbursementUserDTO> page) {
        IPage<AssessReimbursementUserDTO> list = service.findList(page,reimbursementUser);
        return ResponseEntity.ok(list);
    }

    /**
     * 保存业务报销编码
     * @return
     */
    @ApiLog("保存报销人员信息")
    @PostMapping("save")
    public ResponseEntity save(@RequestBody AssessReimbursementUserDTO reimbursementUser) {
        Map<String, Object> result = service.save(reimbursementUser);
        Boolean success = (Boolean) result.get("success");
        if(success){
            return ResponseEntity.ok(result.get("message"));
        }
        return ResponseEntity.badRequest ().body (result.get("message"));
    }

    /**
     * 批量删除
     * @param ids
     * @return
     */
    @DemoMode
    @ApiLog("批量删除")
    @PreAuthorize("hasAuthority('assess:reimbursementUser:del')")
    @DeleteMapping("delete")
    public ResponseEntity delete(String ids) {
        String idArray[] =ids.split(",");
        Map<String, Object> result = service.delete(Lists.newArrayList(idArray));
        if(!(Boolean) result.get("success")){
            return ResponseEntity.badRequest ().body (result.get("message"));
        }
        return ResponseEntity.ok (result.get("message"));
    }

    /**
     * 初始化部门人员报销数据信息（当年）
     * @return
     */
    @ApiLog("初始化部门人员报销数据信息（当年）")
    @PreAuthorize("hasAnyAuthority('assess:reimbursementUser:add')")
    @PostMapping("initialize")
    public ResponseEntity initialize() {
        Map<String, Object> result = service.initialize("兴光评估");
        Boolean success = (Boolean) result.get("success");
        if(success){
            return ResponseEntity.ok(result.get("message"));
        }
        return ResponseEntity.badRequest ().body (result.get("message"));
    }


    /**
     * 查询报销信息
     * @return
     */
    @ApiLog("查询业务报销信息列表")
    @PreAuthorize("hasAuthority('assess:reimbursementBusiness:view')")
    @GetMapping("findListByUserId")
    public ResponseEntity findListByUserId(AssessReimbursementBusinessReturnDTO reimbursementBusinessReturnDTO) {
        if(StringUtils.isBlank(reimbursementBusinessReturnDTO.getUserId())){
            return ResponseEntity.badRequest ().body ("请先选择报销人员信息");
        }

        //根据id查询人员填报的报销信息
        AssessReimbursementUserDTO reimbursementUserDTO = service.queryById(reimbursementBusinessReturnDTO.getUserId());
        if(null == reimbursementUserDTO){
            return ResponseEntity.badRequest ().body ("报销人员信息无效，请重新选择");
        }
        AssessReimbursementBusinessReturnDTO info = service.findListByUserId(reimbursementUserDTO,reimbursementBusinessReturnDTO);
        info.setUserId(reimbursementBusinessReturnDTO.getUserId());
        return ResponseEntity.ok(info);
    }
    /**
     * 下载导入用户数据模板
     *
     * @param response
     * @return
     */
    @PreAuthorize ("hasAnyAuthority('assess:reimbursementUser:import')")
    @GetMapping("import/template")
    @ApiOperation(value = "下载模板")
    public void importFileTemplate(HttpServletResponse response) {
        String fileName = "报销人员数据导入模板.xlsx";
        List<ReimbursementUserImportDTO> list = Lists.newArrayList();
        EasyPoiUtil.exportExcel ( list, "报销人员数据",  "报销人员数据", ReimbursementUserImportDTO.class, fileName, response );
    }

    /**
     * 导入用户数据
     *
     * @return
     */
    @DemoMode
    @PreAuthorize("hasAnyAuthority('assess:reimbursementUser:import')")
    @PostMapping("import")
    @ApiOperation(value = "导入员工报销信息excel")
    public ResponseEntity importFile(MultipartFile file) {
        try {
            int successNum = 0;
            int failureNum = 0;
            StringBuilder failureMsg = new StringBuilder ( );
            List <AssessReimbursementUserDTO> list = EasyPoiUtil.importExcel ( file, 1, 1, AssessReimbursementUserDTO.class );
            List<AssessReimbursementUserDTO> saveList = Lists.newArrayList();
            Map<String, Object> resultMap = new HashMap<>();
            for (AssessReimbursementUserDTO user : list) {
                try {
                    //根据部门和人员名称查询人员信息
                    List<UserDTO> userByNameAndOfficeName = service.getUserByNameAndOfficeName(user);
                    if(userByNameAndOfficeName.size()>1){
                        failureMsg.append ( "。 " + user.getOfficeName() + " 部门下的员工 " + user.getUserName() + " 存在多个，无法进行导入;" );
                        failureNum++;
                    }else if(userByNameAndOfficeName.size() == 0){
                        failureMsg.append ( "。 " + user.getOfficeName() + " 部门下的员工 " + user.getUserName() + " 不存在，无法进行导入;" );
                        failureNum++;
                    }else {
                        user.setUserId(userByNameAndOfficeName.get(0).getId());
                        user.setUser(userByNameAndOfficeName.get(0));
                        user.setKeyFlag(user.getYear() + user.getUserId());
                        saveList.add(user);
                        successNum ++;
                    }
                } catch (ConstraintViolationException ex) {
                    failureMsg.append ( "。 " + user.getOfficeName() + " 部门下的员工 " + user.getUserName() + " 导入失败：" );
                    List <String> messageList = BeanValidators.extractPropertyAndMessageAsList ( ex, ": " );
                    for (String message : messageList) {
                        failureMsg.append ( message + "; " );
                        failureNum++;
                    }
                } catch (Exception ex) {
                    failureNum++;
                    failureMsg.append ( "。 " + user.getOfficeName() + " 部门下的员工 " + user.getUserName() + " 导入失败：" + ex.getMessage ( ) );
                }
            }

            if ( failureNum > 0 ) {
                failureMsg.insert ( 0, "，失败 " + failureNum + " 条员工报销数据，导入信息如下：" );
            }

            if(saveList.size()>0){
                resultMap = service.saveList(saveList);

                Boolean success = (Boolean) resultMap.get("success");
                if(!success){
                    return ResponseEntity.badRequest().body ( resultMap.get("message"));
                }
                Object saveCount = resultMap.get("saveCount");
                Object message = resultMap.get("message");
                return ResponseEntity.ok ( "已成功导入 " + saveCount + " 条员工报销数据" + failureMsg+message );
            }
            return ResponseEntity.ok ( "已成功导入 " + successNum + " 条员工报销数据" + failureMsg );
        } catch (Exception e) {
            return ResponseEntity.badRequest().body ( "导入员工报销数据失败！失败信息：" + e.getMessage ( ) );
        }
    }


    /**
     * 导出兴光评估人员信息数据
     * @param reimbursementUserDTO
     * @param page
     * @param response
     * @throws Exception
     */
    @ApiLog("导出兴光评估人员信息数据")
    @GetMapping("export")
    @ApiOperation(value = "导出兴光评估人员信息数据excel")
    public void exportFile(AssessReimbursementUserDTO reimbursementUserDTO, Page <AssessReimbursementUserDTO> page, ExcelOptions options, HttpServletResponse response) {
        String fileName = options.getFilename ( );

        List <AssessReimbursementUserDTO> result;
        if ( ExportMode.current.equals ( options.getMode ( ) ) ) {
            result = service.findList ( page, reimbursementUserDTO ).getRecords();
        } else if ( ExportMode.selected.equals ( options.getMode ( ) ) ) {
            result = service.findList ( page, reimbursementUserDTO ).getRecords().stream ( ).filter ( reimbursement ->
                    options.getSelectIds ( ).contains ( reimbursement.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize ( -1 );
            page.setCurrent ( 0 );
            result = service.findList ( page, reimbursementUserDTO ).getRecords();
        }
        EasyPoiUtil.exportExcel ( result, "兴光评估人员信息数据",  options.getSheetName ( ), AssessReimbursementUserDTO.class, fileName, response );
    }
}
